/// <reference lib="WebWorker" />
import { compileHtml, compileScript, } from '@showroomjs/core';
import wasmPath from 'esbuild-wasm/esbuild.wasm';
import * as esbuild from 'esbuild-wasm/esm/browser';
const esBuildIsReady = esbuild.initialize({
    wasmURL: wasmPath,
    worker: false,
});
self.onmessage = (ev) => {
    const data = ev.data;
    const handleError = (err) => {
        const errString = err instanceof Error ? err.message : String(err);
        const match = errString.match(errorRegex);
        const meta = match
            ? {
                type: 'compilationError',
                line: Number(match[1]),
            }
            : undefined;
        const errorResult = {
            type: 'error',
            error: String(err),
            messageId: data.messageId,
            meta,
        };
        self.postMessage(errorResult);
    };
    const lang = data.lang;
    if (lang === 'html') {
        esBuildIsReady
            .then(() => compileHtml(data.source, esbuild))
            .then((htmlCompileResult) => {
            const result = Object.assign(Object.assign({}, htmlCompileResult), { type: 'success', messageId: data.messageId, lang: data.lang });
            self.postMessage(result);
        })
            .catch(handleError);
    }
    else {
        esBuildIsReady
            .then(() => compileScript(data.source, esbuild, {
            insertRenderIfEndWithJsx: true,
            language: lang,
        }).then((compileResult) => {
            const result = Object.assign(Object.assign({}, compileResult), { type: 'success', messageId: data.messageId, lang: data.lang });
            self.postMessage(result);
        }))
            .catch(handleError);
    }
};
const errorRegex = /Transform failed with \d+ error:\s+<stdin>:(\d+):(\d+):/;
//# sourceMappingURL=compile-worker.js.map