import { DocumentIcon, ExternalLinkIcon, PuzzleIcon, } from '@heroicons/react/outline';
import { HashtagIcon } from '@heroicons/react/solid';
import { isString } from '@showroomjs/core';
import { styled } from '@showroomjs/ui';
import * as React from 'react';
const searchWorker = new Worker(new URL('./search-worker.js', import.meta.url));
let id = 0;
export const getSearchResult = (searchText) => new Promise((fulfill) => {
    const messageId = id++;
    searchWorker.postMessage({
        messageId,
        searchText,
    });
    function handleMessage(ev) {
        const data = ev.data;
        if (messageId === data.messageId) {
            fulfill(formatOptions(data.result));
            searchWorker.removeEventListener('message', handleMessage);
        }
    }
    searchWorker.addEventListener('message', handleMessage);
});
/**
 * map icon string to actual JSX, as React element can't be serialized via web worker
 */
const formatOptions = (options) => options.map((option) => (Object.assign(Object.assign({}, option), { icon: (isString(option.icon) && iconMap[option.icon]) || option.icon })));
const Document = styled(DocumentIcon, {
    width: 20,
    height: 20,
    color: 'inherit',
});
const Hashtag = styled(HashtagIcon, {
    width: 20,
    height: 20,
    color: 'inherit',
});
const Puzzle = styled(PuzzleIcon, {
    width: 20,
    height: 20,
    color: 'inherit',
});
const External = styled(ExternalLinkIcon, {
    width: 20,
    height: 20,
    color: 'inherit',
});
const iconMap = {
    component: React.createElement(Puzzle, { width: 20, height: 20 }),
    section: React.createElement(Hashtag, { width: 20, height: 20 }),
    markdown: React.createElement(Document, { width: 20, height: 20 }),
    link: React.createElement(External, { width: 20, height: 20 }),
};
//# sourceMappingURL=get-search-result.js.map