#!/usr/bin/env node
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const spawn = (0, tslib_1.__importStar)(require("cross-spawn"));
const yargs = (0, tslib_1.__importStar)(require("yargs"));
const path = (0, tslib_1.__importStar)(require("path"));
const pkgJson = require(path.resolve(__dirname, '../../package.json'));
process.on('unhandledRejection', (err) => {
    throw err;
});
yargs
    .scriptName(pkgJson.name)
    .version(pkgJson.version)
    .usage('$0 <cmd> [args]')
    .command('$0', false, () => yargs.showHelp('log'))
    .command('dev', 'Start showroom development server', {
    port: {
        type: 'number',
        describe: 'Port number for the dev server',
        default: 6969,
    },
    config: {
        type: 'string',
        describe: 'Config file name',
        default: 'react-showroom.js',
    },
    measure: {
        type: 'boolean',
        describe: 'Whether to measure webpack build performance',
    },
}, () => spawnScript('dev', process.argv.slice(3)))
    .command('build', 'Build showroom static site', {
    profile: {
        type: 'boolean',
        describe: 'Whether to generate profile file',
    },
    measure: {
        type: 'boolean',
        describe: 'Whether to measure webpack build performance',
    },
}, () => spawnScript('build', process.argv.slice(3)))
    .help().argv;
function spawnScript(scriptName, argv = []) {
    const result = spawn.sync(process.execPath, [require.resolve(`../scripts/${scriptName}`)].concat(argv), {
        stdio: 'inherit',
    });
    if (result.signal) {
        handleSignal(result.signal);
        process.exit(1);
    }
    process.exit(result.status);
}
function handleSignal(signal) {
    if (signal === 'SIGKILL') {
        console.log('The build failed because the process exited too early. ' +
            'This probably means the system ran out of memory or someone called ' +
            '`kill -9` on the process.');
    }
    else if (signal === 'SIGTERM') {
        console.log('The build failed because the process exited too early. ' +
            'Someone might have called `kill` or `killall`, or the system could ' +
            'be shutting down.');
    }
    process.exit(1);
}
//# sourceMappingURL=react-showroom.js.map