"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createSsrWebpackConfig = exports.createClientWebpackConfig = void 0;
const tslib_1 = require("tslib");
const core_1 = require("@showroomjs/core");
const copy_webpack_plugin_1 = (0, tslib_1.__importDefault)(require("copy-webpack-plugin"));
const html_webpack_plugin_1 = (0, tslib_1.__importDefault)(require("html-webpack-plugin"));
const html_webpack_tags_plugin_1 = (0, tslib_1.__importDefault)(require("html-webpack-tags-plugin"));
const path = (0, tslib_1.__importStar)(require("path"));
const docgen = (0, tslib_1.__importStar)(require("react-docgen-typescript"));
const rehype_mdx_title_1 = require("rehype-mdx-title");
const rehype_slug_1 = (0, tslib_1.__importDefault)(require("rehype-slug"));
const remark_frontmatter_1 = (0, tslib_1.__importDefault)(require("remark-frontmatter"));
const remark_gfm_1 = (0, tslib_1.__importDefault)(require("remark-gfm"));
const remark_mdx_frontmatter_1 = require("remark-mdx-frontmatter");
const webpack = (0, tslib_1.__importStar)(require("webpack"));
const webpack_merge_1 = require("webpack-merge");
const create_doc_parser_1 = require("../lib/create-doc-parser");
const create_hash_1 = require("../lib/create-hash");
const generate_showroom_data_1 = require("../lib/generate-showroom-data");
const log_to_stdout_1 = require("../lib/log-to-stdout");
const merge_webpack_config_1 = require("../lib/merge-webpack-config");
const paths_1 = require("../lib/paths");
const pkg_data_1 = require("../lib/pkg-data");
const rehype_code_auto_id_1 = require("../plugins/rehype-code-auto-id");
const rehype_mdx_headings_1 = require("../plugins/rehype-mdx-headings");
const rehype_meta_as_attribute_1 = require("../plugins/rehype-meta-as-attribute");
const create_babel_preset_1 = require("./create-babel-preset");
const VirtualModulesPlugin = require("webpack-virtual-modules");
const MiniCssExtractPlugin = require('mini-css-extract-plugin');
const CssMinimizerPlugin = require('css-minimizer-webpack-plugin');
const WebpackMessages = require('webpack-messages');
const ReactRefreshWebpackPlugin = require('@pmmmwh/react-refresh-webpack-plugin');
const createClientWebpackConfig = (mode, config, { outDir = 'showroom', profileWebpack = false, measure = false } = {}) => {
    const baseConfig = createBaseWebpackConfig(mode, config, {
        ssr: false,
        profile: profileWebpack,
        measure,
    });
    const { webpackConfig: userConfig, prerender: prerenderConfig, assetDir, basePath, theme, html, } = config;
    const isProd = mode === 'production';
    const clientEntry = (0, paths_1.resolveShowroom)('client-dist/app/showroom-client-entry.js');
    const previewEntry = (0, paths_1.resolveShowroom)('client-dist/app/preview-client-entry.js');
    return (0, merge_webpack_config_1.mergeWebpackConfig)((0, webpack_merge_1.merge)(baseConfig, {
        entry: {
            showroom: config.require
                ? config.require.concat(clientEntry)
                : clientEntry,
            preview: config.require
                ? config.require.concat(previewEntry)
                : previewEntry,
        },
        externals: ['crypto'],
        output: {
            path: (0, paths_1.resolveApp)(outDir),
            publicPath: !isProd ? '/' : `${basePath}/`, // need to add trailing slash
        },
        plugins: [
            // workaround as html-webpack-plugin not compatible with ProfilingPlugin. See https://github.com/jantimon/html-webpack-plugin/issues/1652.
            ...(profileWebpack
                ? []
                : [
                    new html_webpack_plugin_1.default({
                        template: (0, paths_1.resolveShowroom)('html-template/showroom.html'),
                        templateParameters: {
                            favicon: theme.favicon,
                            resetCss: theme.resetCss,
                            backgroundColor: theme.colors['primary-800'],
                            linkManifest: theme.manifest && isProd,
                            basePath,
                        },
                        minify: isProd && {
                            collapseWhitespace: true,
                            keepClosingSlash: true,
                            removeComments: true,
                            ignoreCustomComments: [/SSR-/],
                            removeRedundantAttributes: true,
                            removeScriptTypeAttributes: true,
                            removeStyleLinkTypeAttributes: true,
                            useShortDoctype: true,
                        },
                        chunks: ['showroom'],
                        inject: false,
                    }),
                    html.showroom
                        ? new html_webpack_tags_plugin_1.default({
                            ...html.showroom,
                            files: ['**/index.html'],
                        })
                        : undefined,
                    new html_webpack_plugin_1.default({
                        filename: '_preview.html',
                        template: (0, paths_1.resolveShowroom)('html-template/preview.html'),
                        templateParameters: {
                            title: theme.title,
                            resetCss: theme.resetCss,
                            prerender: mode === 'production' && !!prerenderConfig,
                        },
                        minify: isProd && {
                            collapseWhitespace: true,
                            keepClosingSlash: true,
                            removeComments: true,
                            ignoreCustomComments: [/SSR-/],
                            removeRedundantAttributes: true,
                            removeScriptTypeAttributes: true,
                            removeStyleLinkTypeAttributes: true,
                            useShortDoctype: true,
                        },
                        inject: false,
                        chunks: ['preview'],
                    }),
                    html.preview
                        ? new html_webpack_tags_plugin_1.default({
                            ...html.preview,
                            files: ['**/_preview.html'],
                        })
                        : undefined,
                ]),
            measure
                ? undefined // don't prettify output message when measuring to avoid important info get cleared
                : new WebpackMessages({
                    name: 'showroom',
                    logger: log_to_stdout_1.logToStdout,
                }),
            isProd && assetDir
                ? new copy_webpack_plugin_1.default({
                    patterns: [
                        {
                            from: path.posix.join(assetDir.replace(/\\/g, '/'), '**/*'),
                            to: (0, paths_1.resolveApp)(outDir),
                            context: assetDir,
                            globOptions: {
                                ignore: ['**/*.html'],
                            },
                        },
                    ],
                })
                : undefined,
            theme.serviceWorker && isProd
                ? new (require('workbox-webpack-plugin').InjectManifest)({
                    swSrc: (0, paths_1.resolveShowroom)('client/service-worker/_showroom-service-worker.ts'),
                    exclude: [/.wasm$/, /.map$/, /.html$/],
                })
                : undefined,
        ].filter(core_1.isDefined),
    }), userConfig, mode);
};
exports.createClientWebpackConfig = createClientWebpackConfig;
const createSsrWebpackConfig = (mode, config, { outDir = 'showroom', profileWebpack = false, measure = false } = {}) => {
    const baseConfig = createBaseWebpackConfig(mode, config, {
        ssr: true,
        profile: profileWebpack,
        measure,
    });
    const showroomServer = (0, paths_1.resolveShowroom)('client-dist/app/showroom-server-entry.js');
    const previewServer = (0, paths_1.resolveShowroom)('client-dist/app/preview-server-entry.js');
    return (0, merge_webpack_config_1.mergeWebpackConfig)((0, webpack_merge_1.merge)(baseConfig, {
        entry: {
            ...(config.require ? { requireConfig: config.require } : {}),
            prerender: showroomServer,
            previewPrerender: previewServer,
        },
        output: {
            path: (0, paths_1.resolveApp)(`${outDir}/server`),
            filename: '[name].js',
            library: {
                type: 'commonjs',
            },
        },
        externalsPresets: { node: true },
        target: 'node14.17',
        plugins: measure
            ? [] // don't prettify output message when measuring to avoid important info get cleared
            : [
                new WebpackMessages({
                    name: 'ssr',
                    logger: log_to_stdout_1.logToStdout,
                }),
            ],
    }), config.webpackConfig, mode);
};
exports.createSsrWebpackConfig = createSsrWebpackConfig;
const createBaseWebpackConfig = (mode, config, options) => {
    const { prerender: prerenderConfig, css, url, basePath, theme, sections, imports, wrapper, docgen: docgenConfig, debug, cacheDir, example: exampleConfig, componentsEntry, search, compilerOptions, } = config;
    const isProd = mode === 'production';
    const isDev = mode === 'development';
    const componentTypeParser = docgen.withCustomConfig(docgenConfig.tsconfigPath, {
        shouldRemoveUndefinedFromOptional: true,
    });
    const docParser = (0, create_doc_parser_1.createDocParser)({
        tsconfigPath: docgenConfig.tsconfigPath,
        parserOptions: docgenConfig.options,
    });
    const generated = (0, generate_showroom_data_1.generateSectionsAndImports)(sections, {
        skipEmptyComponent: config.skipEmptyComponent,
        enableAdvancedEditor: exampleConfig.enableAdvancedEditor,
    });
    const virtualModules = new VirtualModulesPlugin({
        // create a virtual module that consists of parsed code blocks
        // so we can pregenerate during build time for better SSR
        [(0, paths_1.resolveShowroom)('node_modules/react-showroom-codeblocks.js')]: (0, generate_showroom_data_1.generateCodeblocksData)(sections),
        // a virtual module that consists of all the sections and component metadata.
        [(0, paths_1.resolveShowroom)('node_modules/react-showroom-sections.js')]: generated.sections,
        [(0, paths_1.resolveShowroom)('node_modules/react-showroom-wrapper.js')]: (0, generate_showroom_data_1.generateWrapper)(wrapper),
        [(0, paths_1.resolveShowroom)('node_modules/react-showroom-all-imports.js')]: generated.allImports,
        [(0, paths_1.resolveShowroom)('node_modules/react-showroom-all-components.js')]: (0, generate_showroom_data_1.generateAllComponents)(sections),
        [(0, paths_1.resolveShowroom)('node_modules/react-showroom-all-components-docs.js')]: (0, generate_showroom_data_1.generateAllComponentsDocs)(sections),
        [(0, paths_1.resolveShowroom)('node_modules/react-showroom-comp-metadata.js')]: (0, generate_showroom_data_1.generateAllComponentsPaths)(sections),
        [(0, paths_1.resolveShowroom)('node_modules/react-showroom-doc-placeholder.js')]: (0, generate_showroom_data_1.generateDocPlaceHolder)(exampleConfig.placeholder),
        [(0, paths_1.resolveShowroom)('node_modules/react-showroom-index.js')]: (0, generate_showroom_data_1.generateSearchIndex)(sections, search.includeHeadings),
    });
    const babelPreset = (0, create_babel_preset_1.createBabelPreset)(mode);
    const codeBlocksOptions = {
        filter: (code) => !(0, core_1.isString)(code.meta) || !code.meta.includes('static'),
    };
    const docsCodeBlocksOptions = {
        filter: (code) => (0, core_1.isString)(code.meta) && code.meta.includes('live'),
    };
    return {
        mode,
        resolve: {
            extensions: paths_1.moduleFileExtensions.map((ext) => `.${ext}`),
        },
        output: {
            filename: isProd ? '_assets/js/[name].[contenthash:8].js' : '[name].js',
            chunkFilename: isProd
                ? '_assets/js/[name].[contenthash:8].js'
                : '[name].js',
            assetModuleFilename: '_assets/media/[name]-[contenthash][ext][query]',
            clean: isProd,
        },
        module: {
            rules: [
                {
                    test: /.js$/,
                    resourceQuery: /showroomCompProp/,
                    use: [
                        {
                            loader: 'showroom-all-component-prop-loader',
                            options: {
                                parse: (sources) => componentTypeParser.parse(sources).map((doc) => Object.assign({}, doc, {
                                    id: (0, create_hash_1.createHash)(doc.filePath),
                                })),
                                dts: exampleConfig.enableAdvancedEditor &&
                                    componentsEntry &&
                                    ((0, core_1.isDefined)(componentsEntry.dts)
                                        ? (0, core_1.isString)(componentsEntry.dts)
                                            ? path.resolve(paths_1.paths.appPath, componentsEntry.dts)
                                            : componentsEntry.dts
                                        : path.resolve(cacheDir, 'dts', `${path.parse(componentsEntry.path).name}.d.ts`)),
                                debug,
                            },
                        },
                    ],
                },
                {
                    test: /\.(js|jsx|ts|tsx)$/,
                    resourceQuery: /showroomCompile/,
                    use: [
                        {
                            loader: require.resolve('babel-loader'),
                            options: {
                                presets: [() => babelPreset],
                                plugins: isDev
                                    ? [require.resolve('react-refresh/babel')]
                                    : undefined,
                                babelrc: false,
                                configFile: false,
                            },
                        },
                    ],
                },
                {
                    test: /\.(ts|tsx)$/,
                    resourceQuery: /docgen/,
                    use: [
                        {
                            loader: 'showroom-docgen-typescript-loader',
                            options: {
                                parser: docParser,
                                debug,
                            },
                        },
                    ],
                },
                {
                    test: /\.(ts|tsx)$/,
                    resourceQuery: {
                        not: [/raw/, /docgen/],
                    },
                    use: [
                        {
                            loader: require.resolve('babel-loader'),
                            options: {
                                presets: [() => babelPreset],
                                plugins: isDev
                                    ? [require.resolve('react-refresh/babel')]
                                    : undefined,
                            },
                        },
                    ],
                },
                {
                    test: /\.mdx?$/,
                    oneOf: [
                        {
                            resourceQuery: /showroomFrontmatter/,
                            use: [
                                {
                                    loader: 'showroom-frontmatter-loader',
                                },
                            ],
                        },
                        {
                            resourceQuery: /headings/,
                            use: [
                                {
                                    loader: 'showroom-remark-headings-loader',
                                },
                            ],
                        },
                        {
                            resourceQuery: /showroomRemarkCodeblocks/,
                            use: [
                                {
                                    loader: 'showroom-remark-codeblocks-loader',
                                    options: codeBlocksOptions,
                                },
                            ],
                        },
                        {
                            resourceQuery: /showroomRemarkDocCodeblocks/,
                            use: [
                                {
                                    loader: 'showroom-remark-codeblocks-loader',
                                    options: docsCodeBlocksOptions,
                                },
                            ],
                        },
                        exampleConfig.enableAdvancedEditor
                            ? {
                                resourceQuery: /showroomRemarkImportsDts/,
                                use: [
                                    {
                                        loader: 'showroom-remark-codeblocks-dts-loader',
                                        options: {
                                            imports,
                                        },
                                    },
                                    {
                                        loader: 'showroom-remark-codeblocks-loader',
                                        options: codeBlocksOptions,
                                    },
                                ],
                            }
                            : undefined,
                        {
                            resourceQuery: /showroomRemarkImports/,
                            use: [
                                {
                                    loader: 'showroom-remark-codeblocks-imports-loader',
                                    options: {
                                        imports,
                                    },
                                },
                                {
                                    loader: 'showroom-remark-codeblocks-loader',
                                    options: codeBlocksOptions,
                                },
                            ],
                        },
                        exampleConfig.enableAdvancedEditor
                            ? {
                                resourceQuery: /showroomRemarkDocImportsDts/,
                                use: [
                                    {
                                        loader: 'showroom-remark-codeblocks-dts-loader',
                                        options: {
                                            imports,
                                        },
                                    },
                                    {
                                        loader: 'showroom-remark-codeblocks-loader',
                                        options: docsCodeBlocksOptions,
                                    },
                                ],
                            }
                            : undefined,
                        {
                            resourceQuery: /showroomRemarkDocImports/,
                            use: [
                                {
                                    loader: 'showroom-remark-codeblocks-imports-loader',
                                    options: {
                                        imports,
                                    },
                                },
                                {
                                    loader: 'showroom-remark-codeblocks-loader',
                                    options: docsCodeBlocksOptions,
                                },
                            ],
                        },
                        {
                            resourceQuery: {
                                not: [/raw/],
                            },
                            use: [
                                {
                                    loader: require.resolve('babel-loader'),
                                    options: {
                                        presets: [() => babelPreset],
                                        plugins: isProd
                                            ? undefined
                                            : [require.resolve('react-refresh/babel')],
                                        babelrc: false,
                                        configFile: false,
                                    },
                                },
                                {
                                    loader: require.resolve('xdm/webpack.cjs'),
                                    options: {
                                        rehypePlugins: [
                                            rehype_slug_1.default,
                                            rehype_meta_as_attribute_1.rehypeMetaAsAttribute,
                                            rehype_mdx_title_1.rehypeMdxTitle,
                                            rehype_code_auto_id_1.rehypeCodeAutoId,
                                            rehype_mdx_headings_1.rehypeMdxHeadings,
                                        ],
                                        remarkPlugins: [
                                            remark_frontmatter_1.default,
                                            [remark_mdx_frontmatter_1.remarkMdxFrontmatter, { name: 'frontmatter' }],
                                            remark_gfm_1.default,
                                        ],
                                    },
                                },
                            ],
                        },
                    ].filter(core_1.isDefined),
                },
                {
                    test: /\.wasm$/,
                    type: 'asset/resource',
                },
                {
                    resourceQuery: /raw/,
                    type: 'asset/source',
                },
                {
                    test: /\.css$/,
                    // if app don't need CSS, we still need to handle css in @showroomjs/device-frames
                    ...(css.enabled
                        ? {}
                        : {
                            include: [/node_modules\/@showroomjs\/device-frames/],
                        }),
                    sideEffects: true,
                    use: [
                        isProd
                            ? {
                                loader: MiniCssExtractPlugin.loader,
                                options: {
                                    emit: !options.ssr,
                                },
                            }
                            : require.resolve('style-loader'),
                        {
                            loader: require.resolve('css-loader'),
                            options: {
                                importLoaders: css.usePostcss ? 1 : 0,
                                modules: {
                                    auto: true,
                                    localIdentName: isProd
                                        ? '[hash:base64]'
                                        : '[path][name]__[local]',
                                },
                            },
                        },
                        css.usePostcss
                            ? {
                                loader: require.resolve('postcss-loader'),
                                options: {
                                    sourceMap: isProd,
                                    postcssOptions: {
                                        config: paths_1.paths.appPostcssConfig,
                                    },
                                },
                            }
                            : undefined,
                    ].filter(core_1.isDefined),
                },
            ].filter(core_1.isDefined),
        },
        resolveLoader: {
            modules: ['node_modules', path.resolve(__dirname, '../webpack-loader')],
        },
        devtool: isProd ? 'source-map' : 'cheap-module-source-map',
        cache: cacheDir
            ? {
                type: 'filesystem',
                name: `react-showroom-${mode}-${options.ssr ? 'ssr' : 'client'}${prerenderConfig ? '-prerender' : ''}`,
                version: [
                    pkg_data_1.pkgData.version,
                    (0, create_hash_1.createHash)(JSON.stringify(config, (_, value) => typeof value === 'function' ? String(value) : value)),
                ].join('-'),
                buildDependencies: {
                    config: [__filename, paths_1.paths.appShowroomConfig],
                    ...(css.usePostcss
                        ? {
                            css: [paths_1.paths.appPostcssConfig],
                        }
                        : {}),
                },
                cacheDirectory: cacheDir,
            }
            : undefined,
        plugins: [
            isProd
                ? new MiniCssExtractPlugin({
                    filename: '_assets/css/[name].[contenthash].css',
                    chunkFilename: '_assets/css/[name].[contenthash].css',
                })
                : undefined,
            new webpack.EnvironmentPlugin({
                PRERENDER: isProd,
                SSR: options.ssr,
                BASE_PATH: isProd ? basePath : '',
                PRERENDER_EXAMPLE: !!prerenderConfig,
                REACT_SHOWROOM_THEME: theme,
                NODE_ENV: mode,
                EXAMPLE_DIMENSIONS: exampleConfig.dimensions,
                ENABLE_ADVANCED_EDITOR: exampleConfig.enableAdvancedEditor,
                SYNC_STATE_TYPE: exampleConfig.syncStateType,
                SHOW_DEVICE_FRAME: exampleConfig.showDeviceFrame,
                A11Y_CONFIG: exampleConfig.a11y.config,
                SITE_URL: url,
                AUDIENCE_TOGGLE: theme.audienceToggle,
                COMPONENTS_ENTRY_NAME: (componentsEntry && componentsEntry.name) || '',
                COMPILER_OPTIONS: compilerOptions,
                USE_SW: theme.serviceWorker,
            }),
            virtualModules,
            isDev
                ? new ReactRefreshWebpackPlugin({
                    overlay: false,
                })
                : undefined,
            options.profile
                ? new webpack.debug.ProfilingPlugin({
                    outputPath: (0, paths_1.resolveApp)(`showroom-webpack-profile${options.ssr ? '-ssr' : ''}.json`),
                })
                : undefined,
            isProd
                ? new webpack.optimize.MinChunkSizePlugin({
                    minChunkSize: 1000,
                })
                : undefined,
            options.measure
                ? new (require('speed-measure-webpack-plugin'))()
                : undefined,
        ].filter(core_1.isDefined),
        optimization: {
            minimize: !options.ssr && isProd,
            minimizer: options.measure
                ? undefined
                : [
                    '...',
                    new CssMinimizerPlugin(),
                ],
            splitChunks: {
                chunks: 'all',
            },
        },
        performance: {
            hints: false,
        },
        infrastructureLogging: {
            level: debug ? 'info' : isProd ? 'info' : 'none',
        },
        stats: debug ? 'normal' : 'none',
        experiments: {
            backCompat: false, // may improve performance, see https://www.tines.com/blog/understanding-why-our-build-got-15x-slower-with-webpack-5
        },
    };
};
//# sourceMappingURL=create-webpack-config.js.map