"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.mdToCodeBlocks = exports.codeblocks = void 0;
const tslib_1 = require("tslib");
const core_1 = require("@showroomjs/core");
const esbuild = (0, tslib_1.__importStar)(require("esbuild"));
const remark_parse_1 = (0, tslib_1.__importDefault)(require("remark-parse"));
const unified_1 = (0, tslib_1.__importDefault)(require("unified"));
const vfile_1 = (0, tslib_1.__importDefault)(require("vfile"));
const create_hash_1 = require("../lib/create-hash");
const all = 'all';
const codeblocks = (tree, { lang = all, name = 'codeblocks', formatter = (v) => v, filter = () => true, } = {}) => {
    const { children } = tree;
    const result = lang === all
        ? {
            [name]: {},
        }
        : {
            [name]: [],
        };
    let i = -1;
    while (++i < children.length) {
        const child = children[i];
        if (child.type === 'code' && child.value && filter(child)) {
            if (lang === all) {
                child.lang = child.lang || '_';
                // @ts-ignore
                result[name][child.lang] = result[name][child.lang] || [];
                // @ts-ignore
                result[name][child.lang].push(formatter(child.value));
            }
            else {
                if (child.lang === lang) {
                    // @ts-ignore
                    result[name].push(formatter(child.value));
                }
            }
        }
    }
    // @ts-ignore
    return result;
};
exports.codeblocks = codeblocks;
const parser = (0, unified_1.default)().use(remark_parse_1.default);
const mdToCodeBlocks = async (mdSource, filter) => {
    const tree = parser.parse((0, vfile_1.default)(mdSource));
    const blocks = (0, exports.codeblocks)(tree, {
        filter,
    }).codeblocks;
    const result = {};
    for (const language of Object.keys(blocks)) {
        const lang = language;
        if (core_1.SUPPORTED_LANGUAGES.includes(lang)) {
            for (const code of blocks[language]) {
                try {
                    if (lang === 'html') {
                        const htmlCompileResult = await (0, core_1.compileHtml)(code, esbuild);
                        result[code] = {
                            ...htmlCompileResult,
                            type: 'success',
                            messageId: -1,
                            initialCodeHash: (0, create_hash_1.createHash)(code),
                            features: [],
                            lang,
                        };
                    }
                    else {
                        const postTranspileResult = await (0, core_1.compileScript)(code, esbuild, {
                            insertRenderIfEndWithJsx: true,
                            language: lang,
                        });
                        result[code] = {
                            ...postTranspileResult,
                            type: 'success',
                            messageId: -1,
                            initialCodeHash: (0, create_hash_1.createHash)(code),
                            lang,
                        };
                    }
                }
                catch (err) {
                    console.error(err);
                }
            }
        }
    }
    return result;
};
exports.mdToCodeBlocks = mdToCodeBlocks;
//# sourceMappingURL=codeblocks.js.map