"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateDts = void 0;
const tslib_1 = require("tslib");
const core_1 = require("@showroomjs/core");
const fs = (0, tslib_1.__importStar)(require("fs-extra"));
const path = (0, tslib_1.__importStar)(require("path"));
const tsup = (0, tslib_1.__importStar)(require("tsup"));
const log_to_stdout_1 = require("./log-to-stdout");
const paths_1 = require("./paths");
const generateDts = async ({ docgen, componentsEntry, cacheDir }, watch) => {
    if (componentsEntry &&
        componentsEntry.path &&
        !(0, core_1.isDefined)(componentsEntry.dts)) {
        (0, log_to_stdout_1.logToStdout)(`Generating d.ts for ${componentsEntry.path}...`);
        const name = path.parse(componentsEntry.path).name;
        const outDir = path.resolve(cacheDir, 'dts');
        if (!watch) {
            await fs.remove(outDir);
        }
        await tsup.build({
            entryPoints: [(0, paths_1.resolveApp)(componentsEntry.path)],
            outDir,
            watch,
            clean: !watch,
            silent: true,
            dts: {
                only: true,
                resolve: true,
            },
            tsconfig: path.isAbsolute(docgen.tsconfigPath)
                ? path.relative(paths_1.paths.appPath, docgen.tsconfigPath)
                : docgen.tsconfigPath,
            config: false,
        });
        await waitFileExist(path.resolve(cacheDir, 'dts', `${name}.d.ts`));
        (0, log_to_stdout_1.logToStdout)((0, log_to_stdout_1.green)(`Generated d.ts for ${componentsEntry.path}.`));
    }
};
exports.generateDts = generateDts;
async function waitFileExist(path, timeout = 20000) {
    let totalTime = 0;
    let checkTime = timeout / 10;
    return await new Promise((resolve) => {
        const timer = setInterval(function () {
            totalTime += checkTime;
            let fileExists = fs.existsSync(path);
            if (fileExists || totalTime >= timeout) {
                clearInterval(timer);
                resolve(fileExists);
            }
        }, checkTime);
    });
}
//# sourceMappingURL=generate-dts.js.map