"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getImportsAttach = void 0;
const core_1 = require("@showroomjs/core");
const paths_1 = require("./paths");
const getClientImportMap = (imports) => imports.reduce((result, importConfig) => {
    if (typeof importConfig === 'string') {
        return {
            ...result,
            [importConfig]: {
                name: importConfig,
                varName: (0, core_1.getSafeName)(importConfig),
                path: importConfig,
            },
        };
    }
    const { name, path } = importConfig;
    const varName = (0, core_1.getSafeName)(name);
    return {
        ...result,
        [name]: {
            name,
            varName,
            path: isPackage(path) ? path : (0, paths_1.resolveApp)(path),
        },
    };
}, {});
const isPackage = (pathName) => /^[a-z\-]+$/.test(pathName);
const getImportsAttach = (importConfigs) => {
    const importMap = getClientImportMap(importConfigs);
    return `export const imports = {};
${Object.values(importMap)
        .map(({ varName, path }) => `import * as ${varName} from '${path}';`)
        .join('\n')}
${Object.values(importMap)
        .map(({ varName }) => `imports.${varName} = ${varName};`)
        .join('\n')}
`;
};
exports.getImportsAttach = getImportsAttach;
//# sourceMappingURL=get-client-import-map.js.map