"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.startDevServer = void 0;
const tslib_1 = require("tslib");
// Do this as the first thing so that any code reading it knows the right env.
process.env.BABEL_ENV = 'development';
process.env.NODE_ENV = 'development';
const path_1 = (0, tslib_1.__importDefault)(require("path"));
const webpack_1 = (0, tslib_1.__importDefault)(require("webpack"));
const webpack_dev_server_1 = (0, tslib_1.__importDefault)(require("webpack-dev-server"));
const yargs_1 = require("yargs");
const create_webpack_config_1 = require("../config/create-webpack-config");
const generate_dts_1 = require("../lib/generate-dts");
const get_config_1 = require("../lib/get-config");
const log_to_stdout_1 = require("../lib/log-to-stdout");
const prepare_url_1 = require("../lib/prepare-url");
const { openBrowser } = require(path_1.default.resolve(__dirname, '..', '..', 'open-browser', 'open-browser.js'));
async function startDevServer(userConfig, configFile, measure) {
    (0, log_to_stdout_1.logToStdout)('Starting dev server...');
    const config = (0, get_config_1.getConfig)('development', configFile, userConfig);
    const { devServerPort, assetDir, example } = config;
    if (example.enableAdvancedEditor) {
        await (0, generate_dts_1.generateDts)(config, true);
    }
    const HOST = '0.0.0.0';
    const PORT = Number(yargs_1.argv.port ?? process.env.PORT ?? devServerPort);
    const webpackConfig = (0, create_webpack_config_1.createClientWebpackConfig)('development', config, {
        measure,
    });
    const devServerOptions = Object.assign({
        port: PORT,
        host: HOST,
        client: {
            logging: 'none',
        },
        hot: true,
        historyApiFallback: {
            rewrites: [
                { from: /^\/_preview/, to: '/_preview.html' },
                { from: /./, to: '/index.html' },
            ],
        },
    }, assetDir
        ? {
            static: {
                directory: assetDir,
                watch: true,
            },
        }
        : {});
    const compiler = (0, webpack_1.default)(webpackConfig);
    const server = new webpack_dev_server_1.default(devServerOptions, compiler);
    await server.start();
    const urls = (0, prepare_url_1.prepareUrls)('http', HOST, PORT);
    openBrowser(urls.localUrlForBrowser);
    (0, log_to_stdout_1.logToStdout)(`Dev server available at ${urls.localUrlForTerminal}`);
    return server;
}
exports.startDevServer = startDevServer;
//# sourceMappingURL=start-dev-server.js.map