"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@showroomjs/core");
const get_type_definitions_for_pkg_1 = require("../lib/get-type-definitions-for-pkg");
const paths_1 = require("../lib/paths");
const showroomRemarkCodeblocksDtsLoader = function (_, __, meta) {
    if (!meta) {
        throw new Error('showroom-remark-codeblocks-dts-loader must be piped after showroom-remark-codeblocks-loader');
    }
    const processed = meta;
    const { imports } = this.getOptions();
    const allImports = imports ? imports.slice() : [];
    Object.keys(processed).forEach((sourceCodeSnippet) => {
        const compiled = processed[sourceCodeSnippet];
        if (compiled) {
            const { importedPackages } = compiled;
            for (const pkgName of importedPackages) {
                if (!allImports.some((importConfig) => (0, core_1.isString)(importConfig)
                    ? importConfig === pkgName
                    : importConfig.name === pkgName)) {
                    allImports.push(pkgName);
                }
            }
        }
    });
    const allTypeDefinitions = (0, core_1.flattenArray)(allImports
        .filter(core_1.isString)
        .map((pkg) => (0, get_type_definitions_for_pkg_1.getTypeDefinitionsForPkg)(pkg, paths_1.paths.appPath)));
    return `${allTypeDefinitions
        .map((def, i) => `import type${i} from '${def.filePath}?raw';`)
        .join('\n')}
    
    export default {
        ${allTypeDefinitions
        .map((def, i) => `'${def.modulePath}': type${i}`)
        .join(',\n')}
    }`;
};
module.exports = showroomRemarkCodeblocksDtsLoader;
//# sourceMappingURL=showroom-remark-codeblocks-dts-loader.js.map